# 
# Copyright (C) 2010-2017 Samuel Abels
# The MIT License (MIT)
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
from Exscript.util        import crypt
from Exscript.stdlib.util import secure_function

@secure_function
def otp(scope, password, seed, seqs):
    """
    Calculates a one-time password hash using the given password, seed, and
    sequence number and returns it.
    Uses the md4/sixword algorithm as supported by TACACS+ servers.

    :type  password: string
    :param password: A password.
    :type  seed: string
    :param seed: A username.
    :type  seqs: int
    :param seqs: A sequence number, or a list of sequence numbers.
    :rtype:  string
    :return: A hash, or a list of hashes.
    """
    return [crypt.otp(password[0], seed[0], int(seq)) for seq in seqs]
